import 'package:flutter/material.dart';
import 'package:flutter/src/foundation/diagnostics.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:advanced_navigator/advanced_navigator.dart';

const String path0 = '/';
const String path1 = '/...';
const String path2 = '/page1';
const String path3 = '/page1/...';
const String path4 = '/page1/{id}/page2';
const String nestedPath1 = '/nestedPage1';

class MockContext implements BuildContext {
  @override
  // TODO: implement debugDoingBuild
  bool get debugDoingBuild => throw UnimplementedError();

  @override
  InheritedWidget dependOnInheritedElement(InheritedElement ancestor, {Object? aspect}) {
    // TODO: implement dependOnInheritedElement
    throw UnimplementedError();
  }

  @override
  T? dependOnInheritedWidgetOfExactType<T extends InheritedWidget>({Object? aspect}) {
    // TODO: implement dependOnInheritedWidgetOfExactType
    throw UnimplementedError();
  }

  @override
  DiagnosticsNode describeElement(String name, {DiagnosticsTreeStyle style = DiagnosticsTreeStyle.errorProperty}) {
    // TODO: implement describeElement
    throw UnimplementedError();
  }

  @override
  List<DiagnosticsNode> describeMissingAncestor({required Type expectedAncestorType}) {
    // TODO: implement describeMissingAncestor
    throw UnimplementedError();
  }

  @override
  DiagnosticsNode describeOwnershipChain(String name) {
    // TODO: implement describeOwnershipChain
    throw UnimplementedError();
  }

  @override
  DiagnosticsNode describeWidget(String name, {DiagnosticsTreeStyle style = DiagnosticsTreeStyle.errorProperty}) {
    // TODO: implement describeWidget
    throw UnimplementedError();
  }

  @override
  void dispatchNotification(Notification notification) {
    // TODO: implement dispatchNotification
  }

  @override
  T? findAncestorRenderObjectOfType<T extends RenderObject>() {
    // TODO: implement findAncestorRenderObjectOfType
    throw UnimplementedError();
  }

  @override
  T? findAncestorStateOfType<T extends State<StatefulWidget>>() {
    // TODO: implement findAncestorStateOfType
    throw UnimplementedError();
  }

  @override
  T? findAncestorWidgetOfExactType<T extends Widget>() {
    // TODO: implement findAncestorWidgetOfExactType
    throw UnimplementedError();
  }

  @override
  RenderObject? findRenderObject() {
    // TODO: implement findRenderObject
    throw UnimplementedError();
  }

  @override
  T? findRootAncestorStateOfType<T extends State<StatefulWidget>>() {
    // TODO: implement findRootAncestorStateOfType
    throw UnimplementedError();
  }

  @override
  InheritedElement? getElementForInheritedWidgetOfExactType<T extends InheritedWidget>() {
    // TODO: implement getElementForInheritedWidgetOfExactType
    throw UnimplementedError();
  }

  @override
  // TODO: implement owner
  BuildOwner? get owner => throw UnimplementedError();

  @override
  // TODO: implement size
  Size? get size => throw UnimplementedError();

  @override
  void visitAncestorElements(bool Function(Element element) visitor) {
    // TODO: implement visitAncestorElements
  }

  @override
  void visitChildElements(ElementVisitor visitor) {
    // TODO: implement visitChildElements
  }

  @override
  // TODO: implement widget
  Widget get widget => throw UnimplementedError();
}

void main() {
  group('AdvancedNavigator.parsePath', () {
    test('Parse empty path /', () {
      final pathGroup0 = AdvancedNavigator.parsePath(path0);
      expect(pathGroup0.length, 0);
      expect(pathGroup0.args, isEmpty);
      expect(pathGroup0.pattern.pattern, '^/\$');
    });

    test('Parse empty nested Path', () {
      final pathGroup1 = AdvancedNavigator.parsePath(path1);
      expect(pathGroup1.length, 0);
      expect(pathGroup1.args, isEmpty);
      expect(pathGroup1.pattern.pattern, '^/');
    });

    test('Parse single item path', () {
      final pathGroup2 = AdvancedNavigator.parsePath(path2);
      expect(pathGroup2.length, 1);
      expect(pathGroup2.args, isEmpty);
      expect(pathGroup2.pattern.pattern, '^/page1\$');
    });
 
    test('Parse single item nested path', () {
      final pathGroup3 = AdvancedNavigator.parsePath(path3);
      expect(pathGroup3.length, 1);
      expect(pathGroup3.args, isEmpty);
      expect(pathGroup3.pattern.pattern, '^/page1');
    });

    test('Parse args path', () {
      final pathGroup4 = AdvancedNavigator.parsePath(path4);
      expect(pathGroup4.length, 3);
      expect(pathGroup4.args[1], 'id');
      expect(pathGroup4.pattern.pattern, '^/page1/[^/]*/page2\$');
    });
  });

  group('DefaultRouterDelegate.setNewRoutePath', () {
    final paths = [path1, path2, path3, path4];
    final routerDelegate = DefaultRouterDelegate(
      context: MockContext(),
      onNestedPathUpdate: (configuration) {},
      paths: { for (var path in paths) AdvancedNavigator.parsePath(path): (_) => [] },
      onGeneratePath: null,
      onUnknownPath: null,
      pages: {},
      onGeneratePage: null,
      onUnknownPage: null,
      onPopPage: null,
      onGenerateRoute: null,
      onUnknownRoute: null,
      transitionDelegate: const DefaultTransitionDelegate<dynamic>(),
      reportsRouteUpdateToEngine: false,
      observers: [],
      restorationScopeId: null,
      tag: null,
    );

    test('Set new route path', () {
      final routeInformation = AdvancedRouteInformation(
        location: path1,
        state: null,
        arguments: null,
      );
      routerDelegate.setNewRoutePath(routeInformation);
      expect(routerDelegate.currentConfiguration.location, path1);

    });
  });
}